/**
 * <copyright>
 * </copyright>
 *
 * $Id$
 */
package com.example.po.provider;


import com.example.po.Item;
import com.example.po.POPackage;

import java.util.Collection;
import java.util.List;

import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.common.command.CompoundCommand;
import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.common.notify.Notification;

import org.eclipse.emf.common.util.ResourceLocator;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.edit.command.SetCommand;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.emf.edit.provider.ComposeableAdapterFactory;
import org.eclipse.emf.edit.provider.IEditingDomainItemProvider;
import org.eclipse.emf.edit.provider.IItemLabelProvider;
import org.eclipse.emf.edit.provider.IItemPropertyDescriptor;
import org.eclipse.emf.edit.provider.IItemPropertySource;
import org.eclipse.emf.edit.provider.IStructuredItemContentProvider;
import org.eclipse.emf.edit.provider.ITreeItemContentProvider;
import org.eclipse.emf.edit.provider.ItemPropertyDescriptor;
import org.eclipse.emf.edit.provider.ItemProviderAdapter;
import org.eclipse.emf.edit.provider.ViewerNotification;

/**
 * This is the item provider adapter for a {@link com.example.po.Item} object.
 * <!-- begin-user-doc -->
 * <!-- end-user-doc -->
 * @generated
 */
public class ItemItemProvider
  extends ItemProviderAdapter
  implements
    IEditingDomainItemProvider,
    IStructuredItemContentProvider,
    ITreeItemContentProvider,
    IItemLabelProvider,
    IItemPropertySource
{
  /**
   * This constructs an instance from a factory and a notifier.
   * <!-- begin-user-doc -->
   * <!-- end-user-doc -->
   * @generated
   */
  public ItemItemProvider(AdapterFactory adapterFactory)
  {
    super(adapterFactory);
  }

  /**
   * Creates a set command for the given item. Special command implementations
   * are used for the price and quantity attributes.
   */
  @Override
  protected Command createSetCommand(EditingDomain domain, EObject owner, EStructuralFeature feature, Object value) // 19.1
  {
    if (feature == POPackage.Literals.ITEM__PRICE)
    {
      return new SetPriceCommand(domain, owner, feature, value);
    }
    else if (feature == POPackage.Literals.ITEM__QUANTITY)
    {
      return new SetQuantityCommand(domain, (Item)owner, feature, value);
    }
    return super.createSetCommand(domain, owner, feature, value);
  }

  /**
   * A command for setting the price of an item. If the item's quantity is
   * 10 or greater, a 10% discount is be applied to the specified price. 
   */
  public static class SetPriceCommand extends SetCommand // 19.1
  {
    public SetPriceCommand(EditingDomain domain, EObject owner, EStructuralFeature feature, Object value)
    {
      super(domain, owner, feature, value);
    }

    @Override
    public void doExecute()
    {
      Item item = (Item)owner;
      if (item.getQuantity() >= 10)
      {
        double newPrice = ((Float)value).floatValue() * 0.9;
        value = new Float(newPrice);
      }
      super.doExecute();
    }
  }

  /**
   * A command for setting the quantity of an item. If the item's quantity changes to (or from)
   * 10 or greater, an additional command is automatically used to apply (or remove) a 10% discount.
   */
  public static class SetQuantityCommand extends CompoundCommand // 19.1
  {
    protected Item item;
    protected Object value;
    protected EditingDomain domain;

    public SetQuantityCommand(EditingDomain domain, Item item, EStructuralFeature feature, Object value)
    {
      super(0);
      this.item = item;
      this.value = value;
      this.domain = domain;
      append(new SetCommand(domain, item, feature, value));
    }

    @Override
    public void execute()
    {
      int oldQuantity = item.getQuantity();
      super.execute();
      int quantity = ((Integer)value).intValue();
      if (oldQuantity < 10 && quantity >= 10 || oldQuantity >= 10 && quantity < 10)
      {
        double newPrice = quantity >= 10 ? item.getPrice() * 0.9 : item.getPrice() / 0.9;
        appendAndExecute(new SetCommand(domain, item, POPackage.Literals.ITEM__PRICE, new Float(newPrice)));
      }
    }
  }

  /**
   * This returns the property descriptors for the adapted class.
   * <!-- begin-user-doc -->
   * <!-- end-user-doc -->
   * @generated
   */
  @Override
  public List<IItemPropertyDescriptor> getPropertyDescriptors(Object object)
  {
    if (itemPropertyDescriptors == null)
    {
      super.getPropertyDescriptors(object);

      addProductNamePropertyDescriptor(object);
      addQuantityPropertyDescriptor(object);
      addPricePropertyDescriptor(object);
    }
    return itemPropertyDescriptors;
  }

  /**
   * This adds a property descriptor for the Product Name feature.
   * <!-- begin-user-doc -->
   * <!-- end-user-doc -->
   * @generated
   */
  protected void addProductNamePropertyDescriptor(Object object)
  {
    itemPropertyDescriptors.add
      (createItemPropertyDescriptor
        (((ComposeableAdapterFactory)adapterFactory).getRootAdapterFactory(),
         getResourceLocator(),
         getString("_UI_Item_productName_feature"),
         getString("_UI_PropertyDescriptor_description", "_UI_Item_productName_feature", "_UI_Item_type"),
         POPackage.Literals.ITEM__PRODUCT_NAME,
         true,
         false,
         false,
         ItemPropertyDescriptor.GENERIC_VALUE_IMAGE,
         null,
         null));
  }

  /**
   * This adds a property descriptor for the Quantity feature.
   * <!-- begin-user-doc -->
   * <!-- end-user-doc -->
   * @generated
   */
  protected void addQuantityPropertyDescriptor(Object object)
  {
    itemPropertyDescriptors.add
      (createItemPropertyDescriptor
        (((ComposeableAdapterFactory)adapterFactory).getRootAdapterFactory(),
         getResourceLocator(),
         getString("_UI_Item_quantity_feature"),
         getString("_UI_PropertyDescriptor_description", "_UI_Item_quantity_feature", "_UI_Item_type"),
         POPackage.Literals.ITEM__QUANTITY,
         true,
         false,
         false,
         ItemPropertyDescriptor.INTEGRAL_VALUE_IMAGE,
         null,
         null));
  }

  /**
   * This adds a property descriptor for the Price feature.
   * <!-- begin-user-doc -->
   * <!-- end-user-doc -->
   * @generated
   */
  protected void addPricePropertyDescriptor(Object object)
  {
    itemPropertyDescriptors.add
      (createItemPropertyDescriptor
        (((ComposeableAdapterFactory)adapterFactory).getRootAdapterFactory(),
         getResourceLocator(),
         getString("_UI_Item_price_feature"),
         getString("_UI_PropertyDescriptor_description", "_UI_Item_price_feature", "_UI_Item_type"),
         POPackage.Literals.ITEM__PRICE,
         true,
         false,
         false,
         ItemPropertyDescriptor.REAL_VALUE_IMAGE,
         null,
         null));
  }

  /**
   * This returns Item.gif.
   * <!-- begin-user-doc -->
   * <!-- end-user-doc -->
   * @generated
   */
  @Override
  public Object getImage(Object object)
  {
    return overlayImage(object, getResourceLocator().getImage("full/obj16/Item"));
  }

  /**
   * This returns the label text for the adapted class.
   * <!-- begin-user-doc -->
   * <!-- end-user-doc -->
   * @generated
   */
  @Override
  public String getText(Object object)
  {
    String label = ((Item)object).getProductName();
    return label == null || label.length() == 0 ?
      getString("_UI_Item_type") :
      getString("_UI_Item_type") + " " + label;
  }

  /**
   * This handles model notifications by calling {@link #updateChildren} to update any cached
   * children and by creating a viewer notification, which it passes to {@link #fireNotifyChanged}.
   * <!-- begin-user-doc -->
   * <!-- end-user-doc -->
   * @generated
   */
  @Override
  public void notifyChanged(Notification notification)
  {
    updateChildren(notification);

    switch (notification.getFeatureID(Item.class))
    {
      case POPackage.ITEM__PRODUCT_NAME:
      case POPackage.ITEM__QUANTITY:
      case POPackage.ITEM__PRICE:
        fireNotifyChanged(new ViewerNotification(notification, notification.getNotifier(), false, true));
        return;
    }
    super.notifyChanged(notification);
  }

  /**
   * This adds {@link org.eclipse.emf.edit.command.CommandParameter}s describing the children
   * that can be created under this object.
   * <!-- begin-user-doc -->
   * <!-- end-user-doc -->
   * @generated
   */
  @Override
  protected void collectNewChildDescriptors(Collection<Object> newChildDescriptors, Object object)
  {
    super.collectNewChildDescriptors(newChildDescriptors, object);
  }

  /**
   * Return the resource locator for this item provider's resources.
   * <!-- begin-user-doc -->
   * <!-- end-user-doc -->
   * @generated
   */
  @Override
  public ResourceLocator getResourceLocator()
  {
    return SimplePOEditPlugin.INSTANCE;
  }

}
